/*
 * A filesystem object accessor.
 *
 * Copyright (C) 2018, 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>

#include <stdio.h>

#include <ext2fs/ext2fs.h>

#include <fsserver/accessor.h>



/* Filesystem object accessor. */

class Fs_accessor : public PagingAccessor
{
private:
  ext2_filsys _fs;
  ext2_ino_t _ino_file;
  int _flags;

  /* Interaction with the file from the block device. */

  ext2_file_t _file;

  /* Semaphore for coordinating access to the file. */

  l4_cap_idx_t _semaphore;

public:
  explicit Fs_accessor(ext2_filsys fs, ext2_ino_t ino_file, int flags,
                       Paging *paging, size_t size=0);

  /* Accessor methods. */

  unsigned int attach();

  unsigned int detach();

  unsigned long get_fileid() { return ext2fs_file_get_inode_num(_file); }

  long flush();

  size_t fstat(char *buffer, size_t length);

  size_t read(size_t start, char *buffer, size_t length);

  size_t write(size_t start, char *buffer, size_t length);

protected:
  /* Data copying methods. */

  size_t read_into(char *buffer, size_t to_transfer);

  size_t write_from(char *buffer, size_t to_transfer);
};
